/*
 * OpenEAS Experlog Application Server
 * Web application development and dynamic template scripting engine
 * Copyright (c) 2004 ExperLog SARL - http://www.experlog.com
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have
 * received a copy of the GNU General Public License along with this program;
 * if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.experlog.actions;

import org.experlog.util.Client;
import org.experlog.base.ESServletRequest;
import org.experlog.base.ESAction;
import org.experlog.base.ESCookie;
import org.experlog.gencode.DynHtml;
import org.experlog.gencode.ESDynHtml;
import org.experlog.util.ESMailer;
import org.experlog.util.AppConfiguration;

import java.io.File;
import java.io.PrintWriter;

/**
 * Send an e-mail
 * Input parameters:
 *  Template: an e-mail template file path, relative to the shop's root dir
 *   (MANDATORY)
 *  From: an e-mail address (OPTIONAL if email.From is defined)
 *  To: an e-mail address (OPTIONAL if email.ManagerEmail is defined)
 *  Subject: the subject (OPTIONAL, default "(No Subject)")
 *  Content_type: the content type & charset (OPTIONAL)
 * Output parameters:
 */
public class SendMail implements ESAction {

	public boolean serverSide() {
		return true;
	}

	public boolean processRequest(
		Client session,
		ESServletRequest req,
		DynHtml page,
		Object info)
		throws Exception {
		String template = req.getParameter("Template");
		if (template == null) {
			req.addParameter("MissingField", "Template");
			req.addParameter("ErrCode", "MISSINGFIELD");
			return false;
		}
		if (!AppConfiguration.isSecurePath(template)) {
			return false;
		}
		String from = req.getParameter("From");
		if (from == null || from.trim().length() < 5) // 5 = length of "a@b.c"
			from = session.getAppConfiguration().get("email.From");
		if (from == null || from.trim().length() < 5) {
			req.addParameter("MissingField", "From");
			req.addParameter("ErrCode", "MISSINGFIELD");
			return false;
		}
		String to = req.getParameter("To");
		if (to == null || to.trim().length() < 5)
			to = session.getAppConfiguration().get("email.ManagerEmail");
		if (to == null || to.trim().length() < 5) {
			req.addParameter("MissingField", "To");
			req.addParameter("ErrCode", "MISSINGFIELD");
			return false;
		}

		DynHtml expander = new ESDynHtml(session, null);
		expander.setServletRequest(req);

		String root = session.getAppConfiguration().getRootDir();
		String fnm;
		if (root.endsWith(File.separator))
			fnm = new String(root + template);
		else
			fnm = new String(root + File.separator + template);

		expander.setInputFile(new File(fnm));

		expander.addDataObject("Cookie", (ESCookie)session.getCookie());
                // ShopConfig is here for backward compatibility...
                // The right name is now AppConfig
		expander.addDataObject("ShopConfig", session.getAppConfiguration());
		expander.addDataObject("AppConfig", session.getAppConfiguration());

		String subject = req.getParameter("Subject");
		if (subject == null)
			subject = new String("(No Subject)");

		ESMailer mailer =
			new ESMailer(session.getAppConfiguration().get("email.MailHost"));

		String mailPort = session.getAppConfiguration().get("email.MailPort");
		if(mailPort != null) {
			try {
				mailer.setPort(Integer.parseInt(mailPort));
			} catch(NumberFormatException e) {
			}
		}

		if ("true"
			.equalsIgnoreCase(
				session.getAppConfiguration().get("email.Legacy")))
			mailer.setLegacy(true);

		mailer.setContentType(req.getParameter("Content_type"));

		java.net.Socket s = null;
		try {
			s = mailer.open();
			PrintWriter p = mailer.beginMessage(s, from, to, subject);

			expander.expand(p);

			mailer.endMessage(p);
			mailer.close(s);
		} catch (Exception ee) {
			System.err.println("SendMail:processRequest Exception " + ee);
			try {
				if (s != null)
					mailer.close(s);
			} catch (Exception ex) {
			}
			req.addParameter("ErrCode", "MAILERROR");
			req.addParameter("Info", ee.getMessage());
			return false; //SendMail failed
		}

		return true;
	}

}
