/*
 * OpenEAS Experlog Application Server
 * Web application development and dynamic template scripting engine
 * Copyright (c) 2004 ExperLog SARL - http://www.experlog.com
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have
 * received a copy of the GNU General Public License along with this program;
 * if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.experlog.actions;

import org.experlog.util.Client;
import org.experlog.base.ESServletRequest;
import org.experlog.base.ESAction;
import org.experlog.base.OpenClient;
import org.experlog.db.ESDbObject;
import org.experlog.gencode.DynHtml;

import java.util.StringTokenizer;

/**
 * Input parameters:
 * Table (MANDATORY)
 * Output parameters:
 */
public class TableInsert implements ESAction {

    public boolean serverSide() { return false; }
    
    public boolean processRequest(Client session, ESServletRequest req,
			DynHtml page, Object info) throws Exception {
	String tbl = req.getParameter("Table");
	if(tbl == null) {
	    req.addParameter("ERRCODE", "MISSINGFIELD");
	    req.addParameter("MissingField", "Table");
	    return false;
	}
	
	/**
	 * Tables on which this action is allowed are spacified in the
	 * configuration file.
	 * Otherwise, this action would be a real security hole !
	 * (users could affect the database tables easily)
	 */
	String allowtables = session.getAppConfiguration().get(
								"actions.ESTableInsert.tables");
	if(allowtables == null) {
	    req.addParameter("ERRCODE", "ACCESSDENIED");
	    return false;
	}
	
	StringTokenizer st = new StringTokenizer(allowtables, ";");
	boolean allow = false;
	while(st.hasMoreTokens()) {
	    allow = tbl.equals(st.nextToken());
	}
	
	if(! allow) {
	    req.addParameter("ERRCODE", "ACCESSDENIED");
	    return false;
	}
	
	ESDbObject dbo = new ESDbObject((OpenClient)session, tbl, null);
	dbo.setFromRequest(req);
	session.updateDB(dbo.insertQuery());
	
	return true;
    }
    
}

