/*
 * OpenEAS Experlog Application Server
 * Web application development and dynamic template scripting engine
 * Copyright (c) 2004 ExperLog SARL - http://www.experlog.com
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have
 * received a copy of the GNU General Public License along with this program;
 * if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.experlog.base;

import org.experlog.gencode.DynHtml;
import org.experlog.gencode.ESDynHtml;
import org.experlog.gencode.UserObject;
import org.experlog.util.Client;
import org.experlog.util.Cookie;
import org.experlog.util.ConnectionPool;
import org.experlog.util.ESCategory;
import org.experlog.util.ESDbMapper;
import org.experlog.util.AppConfiguration;
import org.experlog.util.Store;

import org.apache.log4j.Category;

import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Enumeration;


/**
 * A web client session
 */
public class OpenClient implements Client {

  public ConnectionPool connectionPool_;	// app connection pool
  public String clientId_;			// id of the client

  public ESCookie cookie_ = null;		// cookie of the client's session
  public ESDbMapper dbmapper_ = null;

  public boolean nocomment_ = false;		// for "Generated by openeas" comment in pages
  public long lastaccess_ = 0;
  public long timeoutmillis_;
  public int qryattempts_ = 1;
  public boolean reconnect_on_err_ = false;	// reconnect DB on request error
  public String sqlencodesep_ = null;
  public String sqlencoderules_ = null;

  public  boolean isadm_ = false;
  public String page_ = null;
  public  Hashtable param_ = null;
  public Category logger;

  public Hashtable datasourceConnections_ = null;
  public Hashtable datasourceLogs_ = null;
  public Hashtable connectionCounts_ = null;

  public Hashtable multiPagesObjects_ = null; // list of objects which could be used in several pages

  public OpenClient() { }


  /**
   * constructor of a client, with its clientID, the request,
   * and the jdbc connection
   */
  public OpenClient(String id, HttpServletRequest req,
      ConnectionPool conPool) throws Exception {

    logger = ESCategory.getCategory(ESStore.getInstance().getShopName(), "openeas.client");
    logger.debug("ESClient create a new client");
    clientId_ = id;
    connectionPool_ = conPool;

    String qa = getAppConfiguration().get("jdbc.QryAttempts");
    if (qa != null) {
      try {
        qryattempts_ = Integer.parseInt(qa);
      } catch (NumberFormatException e) {
        qryattempts_ = 1;
      }
    }

    qa = getAppConfiguration().get("jdbc.ReconnectOnError");
    if (qa != null) setReconnect_on_err(qa.equalsIgnoreCase("true"));

    qa = getAppConfiguration().get("internal.NoComment");
    if (qa != null) nocomment_ = qa.equalsIgnoreCase("true");

    resetTimer();
    timeoutmillis_ = getAppConfiguration().getTimeout() * 1000;

    dbmapper_ = new ESDbMapper(getAppConfiguration().get("app.dbmapper"));

    datasourceConnections_ = new Hashtable();
    datasourceLogs_ = new Hashtable();
    connectionCounts_ = new Hashtable();
    multiPagesObjects_ = new Hashtable();

    cookie_ = new ESCookie(this, req,
          getAppConfiguration().useCookies(),
          getAppConfiguration().get("app.persistcookie"),
          getAppConfiguration().getCookieTimeout());

    logger.debug("ESClient a new client is created");
  }

  public Store getStore() {
    return ESStore.getInstance();
  }

  /** TBD */
  public boolean sessionExpired() {
    boolean expired = (System.currentTimeMillis() - lastaccess_) >= timeoutmillis_;
    if (expired) { // Session timeout !
      try { // If there's an "ontimeout" template, execute it
        String template = getAppConfiguration().get("session.ontimeout");
        if (template != null) {
          String fnm = getAppConfiguration().getRootDir();
          if (fnm.endsWith(File.separator))
            fnm += template;
          else
            fnm += File.separator + template;
          fnm = fnm.replace('/', File.separatorChar).replace('\\',
              File.separatorChar);
          DynHtml expander = new ESDynHtml(this, new File(fnm));
          ESServletRequest sr = new ESServletRequest(null, null, this);
          // Set session ID, as if the call came across HTTP with S_ specified
          sr.addParameter("S_", clientId_);
          expander.setServletRequest(sr);
          expander.expand(new PrintWriter(System.out));
        }
      } catch (Exception te) {
        logger.error("ESClient:sessionExpired() - session.ontimeout error:"
            + te);
      }
      getStore().endClient(clientId_); // Application-level cleanup for this
      // session
      //endClient();  // Session-level cleanup
    }
    return expired;
  }

  public void addUserObject(String name, UserObject uobj) {
    if (multiPagesObjects_ != null)          
      multiPagesObjects_.put(name, uobj);
    else
      logger.warn("multiPagesObjects is null, it should not !");
  }

  public UserObject getUserObject(String name) {
    if (multiPagesObjects_ == null) {
      logger.warn("multiPagesObjects is null, it should not !");
      return null;
    }
    return (UserObject) multiPagesObjects_.get(name);
  }

  public Hashtable getUserObjects() {
    return multiPagesObjects_;
  }

  public void closeUserObject(String name) {
    if (multiPagesObjects_ == null) {
      logger.warn("multiPagesObjects is null, it should not !");
    } else {
      UserObject uo = (UserObject) multiPagesObjects_.get(name);
      uo.close();
      uo = null;
      multiPagesObjects_.remove(name);
    }
  }

  public void endClient() {
    if (multiPagesObjects_ == null) {
      logger.warn("multiPagesObjects is null, it should not !");
    } else {
      Enumeration en = multiPagesObjects_.keys();
      while (en.hasMoreElements()) {
        Object key = en.nextElement();
        UserObject val = (UserObject)multiPagesObjects_.get(key);
        val.close();
        val = null;
      }
      en = null;
      multiPagesObjects_.clear();
      multiPagesObjects_ = null;
    }

    if(cookie_ != null) cookie_.clear();
  }

  /**
   * Get the shop configuration
   */
  public AppConfiguration getAppConfiguration() {
    return ESStore.getInstance().getShopConfiguration();
  }

  /**
   * Get the database mapper
   */
  public ESDbMapper getDbMapper() {
    return dbmapper_;
  }

  /**
   * Get the session Id (to recognize who is connected)
   * @return The session id (a String that identifies the client).
   */
  public String getClientId() {
    return clientId_;
  }

  /**
   * Set the session Id (to recognize who is connected)
   * Can be used for customer logon.
   */
  public void setClientId(String id) {
    clientId_ = new String(id);
  }

  /**
   * Get the client info
   * @return The current client info.
   */
  public Cookie getCookie() {
    return cookie_;
  }

  public String getAppName() {
    return ESStore.getInstance().getShopName();
  }

  /**
   * Check if the client is logged in with administrator privileges
   * In this version, there is no support of administration rights.
   */
  public boolean isAdministrator() { return true; }

  // Cache ResultSet columns... due to ODBC bugs (!)
  public boolean cacheRS() {
    try {
      String cac = getAppConfiguration().get("jdbc.cacheRS");
      if (cac == null) return false;
      return cac.trim().toLowerCase().equals("true");
    } catch (Exception e) {
      logger.error("ESClient:cacheRS Exception " + e);
      return false;
    }
  }

  /**
   *
   */
  public boolean multithread() {
    try {
      String mt = getAppConfiguration().get("jdbc.threadSafe");
      if (mt == null) return true;
      return !mt.trim().toLowerCase().equals("false");
    } catch (Exception e) {
      logger.error("ESClient:multithread Exception " + e);
      return true;
    }
  }

  /**
   *
   */
  public void resetTimer() {
    lastaccess_ = System.currentTimeMillis();
  }

  public String sqlEncode(String s) {
    return sqlEncode(null, s);
  }

  /**
   *
   */
  public String sqlEncode(String datasourcename, String s) {

    if (sqlencodesep_ == null) {

      sqlencodesep_ =
        getAppConfiguration().getSqlEncodeSeparator(datasourcename);

      if (sqlencodesep_ == null) sqlencodesep_ = ";";
    }

    if (sqlencoderules_ == null) {

      sqlencoderules_ =
        getAppConfiguration().getSqlEncodeSubst(datasourcename);

      // Rules syntax: c1=s1;c2=s2;...cn=sn (replace char1 with string1, etc...)
      // ; is the default separator (sqlencode.separator to change it)
      // default rules: replace ' with '', and newlines with spaces.
      if (sqlencoderules_ == null)
        sqlencoderules_ = "\'=\'\';\r=\" \";\n=\" \";";
    }

    StringTokenizer st = new StringTokenizer(sqlencoderules_, sqlencodesep_);
    Hashtable subst = new Hashtable();
    while (st.hasMoreTokens()) {
      StringTokenizer st2 = new StringTokenizer(st.nextToken(), "=");
      if (!st2.hasMoreTokens()) continue;
      String k = st2.nextToken();
      if (!st2.hasMoreTokens()) continue;
      String v = st2.nextToken();
      if (v.equals("\" \""))
        v = " ";
      else if (v.equals("\"\"")) v = "";
      subst.put(k, v);
    }

    StringBuffer buf = new StringBuffer();
    for (int i = 0; i < s.length(); i++) {
      char c = s.charAt(i);
      String sub = (String) subst.get(String.valueOf(c));
      if (sub == null)
        buf.append(c);
      else
        buf.append(sub.toString());
    }

    return buf.toString();
  }

  public boolean noComment() {
    return nocomment_;
  }

  public void updateDB(String sql) throws SQLException, Exception {
    updateDB(null, sql);
  }


  public void updateDB(String datasource, String sql)
  throws SQLException, Exception {
    Connection con = null;
    try {
      con = getConnection(datasource);
      logRequest(datasource, sql);
      connectionPool_.updateDB(con, sql);
    } catch (Exception e) {
      throw e;
    } finally {
      closeConnection(con, datasource);
    }

  }



  public synchronized Connection getConnection(String datasourcename) throws Exception {
    logger.debug("OpenClient: getConnection datasourcename=("+datasourcename+")");
    String dsname = null;
    if (datasourcename == null)
      dsname = "__";
    else {
      dsname = datasourcename;
      datasourcename = getAppConfiguration().getName()+"__"+datasourcename;
    }

    Integer ii = (Integer)connectionCounts_.get(dsname);
    int i = 0;
    Connection con = null;
    if (ii != null) 
      i = ii.intValue();

    if (i == 0) {
      con = connectionPool_.connectDb(datasourcename);
      datasourceConnections_.put(dsname, con);
      i = 1;
      connectionCounts_.put(dsname, new Integer(i));
    } else {
      con = (Connection) datasourceConnections_.get(dsname);
      i++;
      connectionCounts_.put(dsname, new Integer(i));
    }
    logger.debug("OpenClient: getConnection total connection ("+dsname+")="+i);
    return con;
  }

  public void logRequest(String dsn, String request) {
    logger.debug("OpenClient: logRequest datasourcename=("+dsn+") request:=("+request+")");
    String dsname = null;
    if (dsn == null)
      dsname = "__";
    else
      dsname = dsn;
    if (datasourceLogs_ == null)
      datasourceLogs_ = new Hashtable();

    DatasourceRequest dr = (DatasourceRequest)datasourceLogs_.remove(dsname);
    if (dr == null)
      dr = new DatasourceRequest();

    dr.addLog(request);
    datasourceLogs_.put(dsname, dr);

    logger.debug("OpenClient: logRequest phrase=("+request+")");
  }

  public Hashtable getLogRequest() {
    return datasourceLogs_;
  }

  public synchronized void closeConnection(Connection con, String datasourcename) throws SQLException {
    logger.debug("OpenClient: closeConnection datasourcename=("+datasourcename+")");
    String dsname = null;
    if (datasourcename == null)
      dsname = "__";
    else
      dsname = datasourcename;

    Integer ii = (Integer)connectionCounts_.get(dsname);
    int i = 0;

    if (ii != null) 
      i = ii.intValue();

    if (i == 1) {
      datasourceConnections_.remove(dsname);
      //if ((con != null) && !(con.isClosed()))
      if (con != null)
        con.close();
      i = 0;
      connectionCounts_.put(dsname, new Integer(i));
    } else if (i > 1) {
      i--;
      connectionCounts_.put(dsname, new Integer(i));
    } else if (i <= 0)
      logger.warn("One more closeConnection + [" + i + "]");

    if (datasourceLogs_ != null) {
      DatasourceRequest dr = (DatasourceRequest)datasourceLogs_.remove(dsname);
      if (dr != null) {
        dr.removeFirstLog();
        if (dr.size() == 0) {
          dr.clear();
        } else
          datasourceLogs_.put(dsname, dr);
      }
    }

    logger.debug("OpenClient: closeConnection total connection ("+dsname+")="+i);
  }


  public ConnectionPool getConnectionPool() {
    return connectionPool_;
  }

  public boolean isNocomment() {
    return nocomment_;
  }

  public long getLastaccess() {
    return lastaccess_;
  }

  public long getTimeoutmillis() {
    return timeoutmillis_;
  }

  public int getQryattempts() {
    return qryattempts_;
  }

  public boolean isReconnect_on_err() {
    return reconnect_on_err_;
  }

  public String getSqlencodesep() {
    return sqlencodesep_;
  }

  public String getSqlencoderules() {
    return sqlencoderules_;
  }

  public boolean isAdm() {
    return isAdministrator();
  }

  public void setNocomment(boolean nocomment) {
    nocomment_ = nocomment;
  }

  public void setLastaccess(long lastaccess) {
    lastaccess_ = lastaccess;
  }

  /** TBD
    public void setTimeoutmillis(long timeoutmillis) {
	timeoutmillis_ = timeoutmillis;
	timer_.setPeriod(timeoutmillis);
    }
   **/

  public void setQryattempts(int qryattempts) {
    qryattempts_ = qryattempts;
  }

  public void setReconnect_on_err(boolean reconnect) {
    reconnect_on_err_ = reconnect;
    connectionPool_.setReconnect_on_err(reconnect);
  }

  public void setSqlencodesep(String sqlencodesep) {
    sqlencodesep_ = sqlencodesep;
  }

  public void setSqlencoderules(String sqlencoderules) {
    sqlencoderules_ = sqlencoderules;
  }

  public void setConnectionPool(ConnectionPool obj) {
    connectionPool_ = obj;
  }

  public void setCookie(Cookie cook) {
    cookie_ = (ESCookie)cook;
  }

  public void setDbmapper(ESDbMapper mapper) {
    dbmapper_ = mapper;
  }

  public void setParam(Hashtable param) {
    param_ = param;
  }

  public Hashtable getParam() {
    return param_;
  }

  public String toString() { 
    return getClientId(); 
  }
}
