/*
 * OpenEAS Experlog Application Server
 * Web application development and dynamic template scripting engine
 * Copyright (c) 2004 ExperLog SARL - http://www.experlog.com
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have
 * received a copy of the GNU General Public License along with this program;
 * if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.experlog.db;

import java.util.Hashtable;

import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.SQLException;

import javax.sql.DataSource;

import org.apache.log4j.Category;

import org.enhydra.jdbc.pool.StandardPoolDataSource;
import org.enhydra.jdbc.pool.GenericPool;
import org.enhydra.jdbc.pool.GenerationObject;

import org.enhydra.jdbc.standard.StandardConnectionPoolDataSource;
import org.enhydra.jdbc.util.Logger;

import org.experlog.util.DataSourceWrapper;
import org.experlog.util.ESCategory;
import org.experlog.util.AppConfiguration;

/**
 * ESDataSourceWrapper is a dataSource wrapper. If there is
 * a loaded datasource in jndi, this datasource is used. in
 * other case, an ESDataSource is created (openeas dataSource)
 * This wrapper only creates (or lookups) the dataSource, and
 * do some delegation operations to the datasource method.
 */
public class ESDataSourceWrapper implements DataSourceWrapper {

	private String dataSourceName_; // the name of the datasource
	private DataSource dataSource_; // the dataSource itself

	public int errorCount = 0;
	public int maxCount = 0;

	private Category logger;
	Logger logWriter;
	private String user_ = null;
	private String password_ = null;

	/**
	 * constructor
	 * no work is done here.
	 */
	public ESDataSourceWrapper(String name) {
	}

	/**
	 * initialize the datasource wrapper,
	 */
	public void init(AppConfiguration shopConfig, DatasourceProperty dsprop)
		throws Exception, SQLException {
		logger = ESCategory.getCategory(shopConfig.getName(), "openeas.db");
		logger.debug("ESDataSourceWrapper create dataSource wrapper");

		logger.debug(
			"ESDataSourceWrapper the datasource property is null, "
				+ "create an ESDataSource object");

		// first, we have to create the datasource itself
		// with default values, like debug, url, password ...

		StandardConnectionPoolDataSource connect =
			new StandardConnectionPoolDataSource();
		connect.setDebug(true);
		//logWriter = new Logger(LogFactory.getLog(name));
		//connect.setLogWriter(logWriter);

		if (dsprop == null) {
			dataSourceName_ = shopConfig.getName();
			user_ = shopConfig.getJdbcUser();
			password_ = shopConfig.getJdbcPassword();
			connect.setUrl(shopConfig.getJdbcUrl());
			connect.setDriverName(shopConfig.getJdbcDriverClass());
			connect.setUser(user_);
			connect.setPassword(password_);

			// second, we create the pool of connection with the previous object
			dataSource_ = new StandardPoolDataSource(connect);
			//((StandardPoolDataSource) dataSource_).setLogWriter(logWriter);
			//((StandardPoolDataSource) dataSource_).setDebug(true);
			((StandardPoolDataSource) dataSource_).setUser(user_);
			((StandardPoolDataSource) dataSource_).setPassword(password_);
			((StandardPoolDataSource) dataSource_).setMinSize(
				shopConfig.getMinConn());
			((StandardPoolDataSource) dataSource_).setMaxSize(
				shopConfig.getMaxConn());
                        ((StandardPoolDataSource) dataSource_).setLifeTime(
				shopConfig.getLifeTime());
                        ((StandardPoolDataSource) dataSource_).setSleepTime(
				shopConfig.getSleepTime());

		} else {
			dataSourceName_ = dsprop.getName();
			user_ = dsprop.getUser();
			password_ = dsprop.getPassword();
			connect.setUrl(dsprop.getUrl());
			connect.setDriverName(dsprop.getDriver());
			connect.setUser(user_);
			connect.setPassword(password_);

			// second, we create the pool of connection with the previous object
			dataSource_ = new StandardPoolDataSource(connect);
			//((StandardPoolDataSource) dataSource_).setLogWriter(logWriter);
			//((StandardPoolDataSource) dataSource_).setDebug(true);
			((StandardPoolDataSource) dataSource_).setUser(user_);
			((StandardPoolDataSource) dataSource_).setPassword(password_);

			((StandardPoolDataSource) dataSource_).setMinSize(
				dsprop.getMinConn());
			((StandardPoolDataSource) dataSource_).setMaxSize(
				dsprop.getMaxConn());
                        ((StandardPoolDataSource) dataSource_).setLifeTime(
				dsprop.getLifeTime());
                        ((StandardPoolDataSource) dataSource_).setSleepTime(
				dsprop.getSleepTime());
		}

		((StandardPoolDataSource) dataSource_).setJdbcTestStmt(
			shopConfig.getJdbcTestStmt());
		((StandardPoolDataSource) dataSource_).setCheckLevelObject(
			shopConfig.getCheckLevelObject());
		((StandardPoolDataSource) dataSource_).setDataSourceName("xsds");

		logger.debug(
			"ESDataSourceWrapper set the logWriter for the datasource");
	}



	/**
	 * get a connection from the datasource
	 */
	synchronized public Connection getConnection()
		throws SQLException, Exception {
		return getConnection(user_, password_);
	}

	/**
	 * get a connection from the datasource with a specific user
	 * and password properties
	 */
	synchronized public Connection getConnection(
		String _user,
		String _password)
		throws SQLException, Exception {

		Connection con = null;

                // we will wait for an object !
                if (getMaxCon() == ((StandardPoolDataSource) dataSource_).getLockedObjectCount())
                        errorCount++;
                        
		con = dataSource_.getConnection(_user, _password);
                
                // now we verify the maximum of attempted connections
                int i = ((StandardPoolDataSource) dataSource_).getLockedObjectCount();
                if (i > maxCount)
                        maxCount = i;
                        
		return con;
	}

	public void setDataSourceName(String name) {
		dataSourceName_ = name;
	}

	public String getDataSourceName() {
		return dataSourceName_;
	}

	public DataSource getDataSource() {
		return dataSource_;
	}

	public int getMinCon() {
		return ((StandardPoolDataSource) dataSource_).getMinSize();
	}

	public int getMaxCon() {
		return ((StandardPoolDataSource) dataSource_).getMaxSize();
	}

	public String getJdbcTestStmt() {
		return ((StandardPoolDataSource) dataSource_).getJdbcTestStmt();
	}

	public int getCheckLevelObject() {
		return ((StandardPoolDataSource) dataSource_).getCheckLevelObject();
	}

	public void setMinCon(int min) {
		try {
			((StandardPoolDataSource) dataSource_).setMinSize(min);
		} catch (Exception e) {
			logger.error("ESDataSourceWrapper:setMinCon error in setMinSize");
		}
	}

	public void setMaxCon(int max) {
		try {
			((StandardPoolDataSource) dataSource_).setMaxSize(max);
		} catch (Exception e) {
			logger.error("ESDataSourceWrapper:setMaxCon error in setMaxSize");
		}
	}

	public void setJdbcTestStmt(String jdbcTestStmt) {
		((StandardPoolDataSource) dataSource_).setJdbcTestStmt(jdbcTestStmt);
	}

	public void setCheckLevelObject(int checkLevelObject) {
		((StandardPoolDataSource) dataSource_).setCheckLevelObject(
			checkLevelObject);
	}

	public void setLogWriter(PrintWriter log) {
		try {
			dataSource_.setLogWriter(log);
		} catch (SQLException e) {
			logger.error("ESDataSourceWrapper:setLogWriter error " + e);
		}
	}

	public void setDebug(boolean debug) {
		((StandardPoolDataSource) dataSource_).setDebug(debug);
	}

	public void setUser(String user) {
		((StandardPoolDataSource) dataSource_).setUser(user);
	}

	public void setPassword(String password) {
		((StandardPoolDataSource) dataSource_).setPassword(password);
	}

	public PrintWriter getLogWriter() {
		try {
			return dataSource_.getLogWriter();
		} catch (SQLException e) {
			logger.error("ESDataSourceWrapper:setLogWriter error " + e);
		}
		return null;
	}

	public boolean isDebug() {
		return ((StandardPoolDataSource) dataSource_).isDebug();
	}

	public String getUser() {
		return ((StandardPoolDataSource) dataSource_).getUser();
	}

	public String getPassword() {
		return ((StandardPoolDataSource) dataSource_).getPassword();
	}

	public String getUrl() {
		return (
			(StandardConnectionPoolDataSource)
				((StandardPoolDataSource) dataSource_).cpds)
			.getUrl();
	}

	public String getDriverName() {
		return (
			(StandardConnectionPoolDataSource)
				((StandardPoolDataSource) dataSource_).cpds)
			.getDriverName();
	}

	public int getMaxCount() {
		return maxCount;
	}

	public int getErrorCount() {
		return errorCount;
	}

	public void cleanup() {
		((StandardPoolDataSource) dataSource_).stopPool();
	}

    public Hashtable getPool() {
	GenericPool gp = ((StandardPoolDataSource) dataSource_).getPool();
	Hashtable hgp = gp.getUnlockedPool();
	System.out.println("gp.getUnlockedPool:"+hgp.toString());
	return hgp;
    }

    public void removeUnlockedObject(GenerationObject obj) {
	GenericPool gp = ((StandardPoolDataSource) dataSource_).getPool();
	gp.removeUnlockedObject(obj);
    }

    public Hashtable getUsedPool() {
	GenericPool gp = ((StandardPoolDataSource) dataSource_).getUsedPool();
	Hashtable hgp = gp.getLockedPool();
	System.out.println("gp.getLockedPool:"+hgp.toString());
	return hgp;
    }

    public void removeLockedObject(GenerationObject obj) {
	GenericPool gp = ((StandardPoolDataSource) dataSource_).getUsedPool();
	gp.removeLockedObject(obj);
    }
}