/*
 * OpenEAS Experlog Application Server
 * Web application development and dynamic template scripting engine
 * Copyright (c) 2004 ExperLog SARL - http://www.experlog.com
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have
 * received a copy of the GNU General Public License along with this program;
 * if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.experlog.util;

import java.net.URLEncoder;
import java.net.URLDecoder;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Vector;

/**
 */
public class ESParamList {

	private Hashtable params_ = null;
	
	
	public ESParamList() {
		params_ = new Hashtable(8);
	}

	public ESParamList(Hashtable h) {
		if (h == null)
			params_ = new Hashtable(8);
		else
			params_ = h;
	}

	public void decode(String plist) {
		if (plist == null || plist.length() <= 0) return;
		StringTokenizer st = new StringTokenizer(plist, "&");
		while (st.hasMoreTokens()) {
			String ptok = st.nextToken();
			int pos = ptok.indexOf('=');
			if (pos > 0 && ptok.length() >= 3) {  //At least "a=b"
				String k = ptok.substring(0, pos);
				String v = ptok.substring(pos + 1);
				
					params_.put(UrlDecode(k), UrlDecode(v));
				
			}
		}
	}

	public String encode() {
		StringBuffer buf = new StringBuffer();
		Enumeration en = params_.keys();
		while (en.hasMoreElements()) {
			Object k = en.nextElement();
			String val = params_.get(k).toString();
			if (val == null || val.equals("")) {
				val = " ";
			}
			
				buf.append(UrlEncode(k.toString()) + "="
						   + UrlEncode(val));
			
			if (en.hasMoreElements()) buf.append("&");
		}
		return buf.toString();
	}

	public void textdecode(String plist, String sep) {
		if (sep == null || sep.length() > 1) sep = "|";
		if (plist == null || plist.length() <= 0) return;
		StringTokenizer st = new StringTokenizer(plist, sep);
		while (st.hasMoreTokens()) {
			String ptok = st.nextToken();
			int pos = ptok.indexOf('=');
			if (pos > 0 && ptok.length() >= 3) {  //At least "a=b"
				String k = ptok.substring(0, pos);
				String v = ptok.substring(pos + 1);
				params_.put(k, v);
			}
		}
	}

	public String textencode(String sep) {
		if (sep == null || sep.length() > 1) sep = "|";
		StringBuffer buf = new StringBuffer();
		Enumeration en = params_.keys();
		while (en.hasMoreElements()) {
			Object k = en.nextElement();
			String val = params_.get(k).toString();
			if (val == null || val.equals("")) {
				val = " ";
			}
			buf.append(k.toString() + "=" + val);
			if (en.hasMoreElements()) buf.append(sep);
		}
		return buf.toString();
	}

  /**
   * Like Hashtable.get()
   */
	public Object get(String k) {
		return params_.get(k);
	}

  /**
   * Like Hashtable.put(), but ignores null values
   */
	public void put(String k, Object v) {
    if(k == null || v == null) return;
		params_.put(k, v);
	}

  /**
   * Like Hashtable.keys()
   */
  public Enumeration keys() {
    return params_.keys();
  }

	public String toString() {
		return params_.toString();
	}

	/**
	 * Encode so that the encoding for 2 ESParamList objects that are "equal"
	 * (that contain the same information) are equal.
	 */
	public String uniqueEncode() {
		Vector v = new Vector(8);
		Enumeration en = params_.keys();
		while (en.hasMoreElements()) {
			String crt = en.nextElement().toString();
			boolean ins = false;
			for (int i = 0; i < v.size(); i++) {
				if (crt.compareTo((String) v.elementAt(i)) <= 0) {
					v.insertElementAt(crt, i);
					ins = true;
					break;
				}
			}
			if (!ins) v.addElement(crt);
		}
		StringBuffer buf = new StringBuffer();
		en = v.elements();
		while (en.hasMoreElements()) {
			Object k = en.nextElement();
			
				buf.append(UrlEncode(k.toString()) + "="
						   + UrlEncode(params_.get(k).toString()));
			
			if (en.hasMoreElements()) buf.append("&");
		}
		return buf.toString();
	}

	public Hashtable toHashtable() {
		return params_;
	}

	private String UrlEncode(String s) {
		try {
			return URLEncoder.encode(s, "UTF-8");
		} catch(Exception e) {
			return s;
		}
        }
	private String UrlDecode(String s) {
		try {
			return URLDecoder.decode(s, "UTF-8");
		} catch(Exception e) {
			return s;
		}
	}

	public static void main(String args[]) {
		ESParamList p = new ESParamList();
		p.put("Firstname", "Pierre Yves");
		p.put("Last Name", "Gibello");
		System.out.println(p.toString());

		String encoded;
		encoded = p.encode();
		System.out.println(encoded);

		p.decode(encoded);
		System.out.println(p.toString());

		encoded = p.uniqueEncode();
		System.out.println(encoded);

		p.decode(encoded);
		System.out.println(p.toString());
	}
}

